program DelphiMRUManager;

uses
  Vcl.Forms,
  Winapi.TlHelp32,
  System.SysUtils,
  Winapi.Windows, Winapi.Messages,
  MainForm in 'MainForm.pas' ,
  USigned in 'USigned.pas';

function ProcessExists(exeFileName: string): Integer;
var
  ContinueLoop: Winapi.Windows.BOOL;
  FSnapshotHandle: Winapi.Windows.THandle;
  FProcessEntry32: Winapi.TlHelp32.TProcessEntry32;
begin
  FSnapshotHandle := Winapi.TlHelp32.CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
  FProcessEntry32.dwSize := System.SizeOf(FProcessEntry32);
  ContinueLoop := Winapi.TlHelp32.Process32First(FSnapshotHandle, FProcessEntry32);
  Result := 0;
  while Integer(ContinueLoop) <> 0 do
  begin
    if ((System.SysUtils.UpperCase(ExtractFileName(FProcessEntry32.szExeFile)) =
      System.SysUtils.UpperCase(ExeFileName)) or (System.SysUtils.UpperCase(FProcessEntry32.szExeFile) =
      System.SysUtils.UpperCase(ExeFileName))) then
    begin
      Inc(Result);
    end;
    ContinueLoop := Process32Next(FSnapshotHandle, FProcessEntry32);
  end;
  Winapi.Windows.CloseHandle(FSnapshotHandle);
end;

function FindWindowByStringInTitle(aString: string): HWND;
var
  NextHandle: Winapi.Windows.HWND;
  NextTitle: array[0..260] of char;
begin
  NextHandle := Winapi.Windows.GetWindow(Application.Handle, GW_HWNDFIRST);
  while NextHandle > 0 do
  begin
    Winapi.Windows.GetWindowText(NextHandle, NextTitle, 255);
    if Pos(aString, StrPas(NextTitle)) <> 0 then
    begin
      Result := NextHandle;
      EXIT;
    end
    else
      NextHandle := Winapi.Windows.GetWindow(NextHandle, GW_HWNDNEXT);
  end;
  Result := 0;
end;

procedure SendProjectFilePathToExistingInstance(const aProjectFilePath: string; const aHandle: HWND);
begin
  var fFileHandle := Winapi.Windows.CreateFileMapping($FFFFFFFF, nil, PAGE_READWRITE, 0, Length(aProjectFilePath) + 1, 'Project');
  if fFileHandle <> 0 then
  begin
    var Mem := Winapi.Windows.MapViewOfFile(fFileHandle, FILE_MAP_WRITE, 0, 0, 0);
    StrCopy(Mem, PChar(aProjectFilePath));
    Winapi.Windows.UnmapViewOfFile(Mem);
    var LHandle := aHandle;
    if LHandle <> 0 then
      Winapi.Windows.SendMessage(LHandle, WM_USER + 6, 0, 0);
    Winapi.Windows.CloseHandle(fFileHandle);
  end;
end;

procedure SetWindowToTheForeGround(const aHandle: HWND);
begin
  if Winapi.Windows.IsIconic(aHandle) then
  begin
    Winapi.Windows.ShowWindow(aHandle, SW_RESTORE);
  end;
  Winapi.Windows.SetForegroundWindow(aHandle);
end;

{$R *.res}

begin
  var PInstances := ProcessExists('DelphiMRUManager.exe');
  if PInstances > 1 then
  begin
    if ParamCount >= 1 then
    begin
      var ThisProjectFilePath: string := '';
      if FindCmdLineSwitch('IDE') then
      begin
        if ParamCount = 2 then
          ThisProjectFilePath := ParamStr(2);
      end
      else
      begin
        if ParamCount = 1 then
          ThisProjectFilePath := ParamStr(1);
      end;

      if ThisProjectFilePath <> '' then
      begin
        if SameText(ExtractFileExt(ThisProjectFilePath), '.DPROJ') then
        begin
          if FileExists(ThisProjectFilePath) then
          begin
            var LHandle := FindWindowByStringInTitle('Delphi MRU Project Manager');
            SendProjectFilePathToExistingInstance(ThisProjectFilePath, LHandle);
            SetWindowToTheForeGround(LHandle);
          end;
        end;
      end;
    end
    else
    begin
      var LHandle := FindWindowByStringInTitle('Delphi MRU Project Manager');
      SetWindowToTheForeGround(LHandle);
    end;

    EXIT;
  end;

  Application.Initialize;
  Application.MainFormOnTaskbar := True;
  Application.CreateForm(TformMain, formMain);
  Application.Run;
end.

